package ch.unibe.piccola.bytecode;

import ch.unibe.piccola.PiccolaException;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import org.apache.bcel.Constants;
import org.apache.bcel.generic.ClassGen;

/**
 * @author  $Id: ByteCodeFactory.java,v 3.5 2003/01/10 13:46:24 kneubuhl Exp $
**/

public class ByteCodeFactory implements Constants, FactoryConstants {

	private InterfaceFactory m_InterfaceFactory;

	private SubClassFactory m_SubClassFactory;

	private ClassFactory m_ClassFactory;

	private ArrayFactory m_ArrayFactory;

	private ClassGen m_ClassGen;

 	public ByteCodeFactory() {
		m_InterfaceFactory = new InterfaceFactory();
		m_SubClassFactory = new SubClassFactory();
		m_ClassFactory = new ClassFactory();
		m_ArrayFactory = new ArrayFactory();
		m_ClassGen = null;
	}

	public void generateClass(String className, String superClassName,
								ArrayList interfaceNameList, int modifiers,
								ArrayList methodDeclarationList) {
		m_ClassGen = m_ClassFactory.generateClass(className, superClassName,
													interfaceNameList, modifiers, methodDeclarationList);
	}

	public void generateClass(String className, String superClassName,
								ArrayList interfaceNameList, int modifiers,
								ArrayList fieldDeclarationList,
								ArrayList methodDeclarationList) {
		m_ClassGen = m_ClassFactory.generateClass(className, superClassName,
													interfaceNameList, modifiers,
													fieldDeclarationList, methodDeclarationList);
	}

	public void generateSubClass(String superClassName) {
		String genClassName = this.generatedClassName(superClassName);
		//TODO: package correct?
		m_ClassGen = m_SubClassFactory.generateSubclass("ch.unibe.piccola.bytecode."+genClassName, superClassName);
	}

	public void generateInterface(String interfaceName) {
		ArrayList interfaceNames = new ArrayList();
		interfaceNames.add(interfaceName);
		generateInterfaces(interfaceNames);
	}

	public void generateInterfaces(ArrayList interfaceNames) {
		String genClassName = this.generatedClassName((String)interfaceNames.get(0));
		//TODO: package correct?
		m_ClassGen = m_InterfaceFactory.implementInterface("ch.unibe.piccola.bytecode."+genClassName,
															"java.lang.Object",
															interfaceNames);
	}

	public void generateArray(String className) {
		m_ClassGen = m_ArrayFactory.createArrayOfClassName(className);
	}

	// ---

	public Class loadClass() {
    	ByteArrayOutputStream out = new ByteArrayOutputStream();
		Class loadedClass = null;
    	try { m_ClassGen.getJavaClass().dump(out); }
    	catch (java.io.IOException e) {
			throw new PiccolaException("Bridge:  Factory Exception: Couldn't dump `" +
										m_ClassGen.getClassName() + "' to stream.");
		}
      	PiccolaClassLoader classLoader = new PiccolaClassLoader();
		try {
			loadedClass = classLoader.define(m_ClassGen.getClassName(), out.toByteArray());
   		  	out.close();
    	}
    	catch (java.io.IOException e) {
			throw new PiccolaException("Bridge:  Factory Exception: Couldn't define `" +
										m_ClassGen.getClassName() + "' in VM.");
		}
		//System.out.println(m_ClassGen.getClassName() + " loaded.");
		return loadedClass;
  	}

	public void saveClass(String path, String fileName) {
		try { m_ClassGen.getJavaClass().dump(path + fileName);}
		catch(java.io.IOException e) {
			throw new PiccolaException("Bridge:  Factory Exception: Couldn't write `" +
										m_ClassGen.getClassName() + "' to disk.");
		}
      	//System.out.println(fileName + " saved to file.");
	}

	public void saveClass() {
		/* Convert full package name into a path
		 * --> stores the generated class in the bridge directory
		 *     simplifies debugging
		 */

		String className = m_ClassGen.getClassName();

		String fileName = this.getClassName(className) + ".class";
		String path = (className.substring(0, className.lastIndexOf('.'))).replace('.','/');

		//System.err.println("Path: " + path + ", file name: " + fileName);

		this.saveClass("build/" + path +"/", fileName);
	}

	// -----------

	private String generatedClassName(String fullClassName) {
		String className = this.getClassName(fullClassName);
		return "Gen" + className;
	}

	private String getClassName(String fullClassName) {
		return fullClassName.substring(fullClassName.lastIndexOf('.')+1, fullClassName.length());
	}

}
