package ch.unibe.piccola.bytecode;

import ch.unibe.piccola.vm.Form;
import ch.unibe.piccola.vm.FormFactory;
import ch.unibe.piccola.vm.PiccolaService;
import ch.unibe.piccola.vm.VMPool;
import ch.unibe.piccola.code.CodeFactory;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import java.lang.reflect.*;
import java.util.ArrayList;

public class ByteCodeFactoryInterfaceTest extends TestCase implements FactoryConstants {

    // test suite

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }


    public static TestSuite suite()
    {
        return new TestSuite(ByteCodeFactoryInterfaceTest.class);
    }


    public ByteCodeFactoryInterfaceTest(String name)
    {
        super(name);
    }

	 // testing environment

    public void setUp()
    {
		bcf = new ByteCodeFactory();
		sf = new CodeFactory();
		host = new VMPool();
		ff = new FormFactory();
	}

	private VMPool host;

	private FormFactory ff;

	private ByteCodeFactory bcf;

	private CodeFactory sf;

    private Form newService(CodeFactory sf)
    {
        return ff.serviceForm(new PiccolaService(host, sf.code(), ff.emptyForm()));
    }

	// tests

	public void testMultipleArgumentInterface() {
		bcf.generateInterface("ch.unibe.piccola.bytecode.MultipleArgumentInterfaceTest");
		assertNotNull(bcf.loadClass());
	}

	public void testArrayInterface() {
		bcf.generateInterface("ch.unibe.piccola.bytecode.ArrayInterfaceTest");
		assertNotNull(bcf.loadClass());
	}

	public void testLangReflectMemberInterface() {
		String pkgName = "java.lang.reflect.";
		bcf.generateInterface(pkgName + "Member");
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		String name = "GMember";
		int modifiers = 8;
		Class declaringClass = null;
		try{
			declaringClass= Class.forName("java.lang.String");
		} catch (ClassNotFoundException cnfe) {cnfe.printStackTrace();}

		sf.emitJavaObject(null, name);
        sf.emitExit(null);
		Form a = newService(sf);
        Form bBindA = a.bind("getName");

		sf.clear();
		sf.emitJavaObject(null, new Integer(modifiers));
		sf.emitExit(null);
		Form b = newService(sf);
        Form bBindB = b.bind("getModifiers");

		sf.clear();
		sf.emitJavaObject(null, declaringClass);
		sf.emitExit(null);
		Form c = newService(sf);
        Form bBindC = c.bind("getDeclaringClass");

        sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form x = newService(sf);
        Form xBindX = x.bind("initialize");

        Form ext = bBindA.extend(bBindB).extend(bBindC).extend(xBindX);

		//System.err.println("Form ext: " + ext);
		// Whoever said that writing tests is fun, was wrong.
		Constructor cstr = null;
		Class par1 = null;
		try {
			par1 = Class.forName(FORM_CLASS);
		} catch (ClassNotFoundException cnfe) {cnfe.printStackTrace();}
		Class[] parameterTypes = {par1};
		try {
			cstr= aClass.getConstructor(parameterTypes);
		} catch (NoSuchMethodException nsme) {nsme.printStackTrace();}
		Object[] initArgs = {ext};

		Object obj = null;
		try {
			obj = cstr.newInstance(initArgs);
		} catch (Exception e) {e.printStackTrace();}

		//Now let's test:
		assertEquals(name, ((Member) obj).getName());
		assertEquals(modifiers, ((Member) obj).getModifiers());
		assertSame(declaringClass, ((Member) obj).getDeclaringClass());
	}


	public void testVoidArgumentInterface() {
		String pkgName = "ch.unibe.piccola.bytecode.";
		bcf.generateInterface(pkgName + "VoidArgumentInterfaceTest");
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		sf.emitJavaObject(null, new Integer(42));
        sf.emitExit(null);
		Form a = newService(sf);
        Form aBindA = a.bind("intAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Boolean(true));
		sf.emitExit(null);
		Form b = newService(sf);
        Form bBindB = b.bind("booleanAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Short((short) 42));
		sf.emitExit(null);
		Form c = newService(sf);
        Form cBindC = c.bind("shortAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Byte((byte) 42));
		sf.emitExit(null);
		Form d = newService(sf);
        Form dBindD = d.bind("byteAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Long((long) 42));
		sf.emitExit(null);
		Form e = newService(sf);
        Form eBindE = e.bind("longAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Double((double) 42));
		sf.emitExit(null);
		Form f = newService(sf);
        Form fBindF = f.bind("doubleAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Float((float) 42));
		sf.emitExit(null);
		Form g = newService(sf);
        Form gBindG = g.bind("floatAvoid");

		sf.clear();
		sf.emitJavaObject(null, new Character('a'));
		sf.emitExit(null);
		Form h = newService(sf);
        Form hBindH = h.bind("charAvoid");

		sf.clear();
		sf.emitJavaObject(null, "Finished!");
		sf.emitExit(null);
		Form i = newService(sf);
        Form iBindI = i.bind("ObjectAvoid");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form j = newService(sf);
        Form jBindJ = j.bind("voidAvoid");

        sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form x = newService(sf);
        Form xBindX = x.bind("initialize");

        Form ext = aBindA.extend(bBindB).extend(cBindC).extend(dBindD);
		ext = ext.extend(eBindE).extend(fBindF).extend(gBindG);
		ext = ext.extend(hBindH).extend(iBindI).extend(jBindJ).extend(xBindX);

		Object obj = this.instantiateInterface(aClass, ext);

		//Now let's test:
		assertEquals(42, ((VoidArgumentInterfaceTest) obj).intAvoid());
		assertEquals(true, ((VoidArgumentInterfaceTest) obj).booleanAvoid());
		assertEquals(42, ((VoidArgumentInterfaceTest) obj).shortAvoid());
		assertEquals(42, ((VoidArgumentInterfaceTest) obj).byteAvoid());
		assertEquals(42, ((VoidArgumentInterfaceTest) obj).longAvoid());
		assertEquals((double) 42, ((VoidArgumentInterfaceTest) obj).doubleAvoid(),1);
		assertEquals((float) 42, ((VoidArgumentInterfaceTest) obj).floatAvoid(),1);
		assertEquals('a', ((VoidArgumentInterfaceTest) obj).charAvoid());
		assertEquals("Finished!", ((VoidArgumentInterfaceTest) obj).ObjectAvoid());

	}

	public void testVoidReturnInterface() {
		String pkgName = "ch.unibe.piccola.bytecode.";
		bcf.generateInterface(pkgName + "VoidReturnInterfaceTest");
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form j = newService(sf);
        Form jBindJ = j.bind("voidAvoid");

		sf.emitEmpty(null);
        sf.emitExit(null);
		Form a = newService(sf);
        Form aBindA = a.bind("voidAint");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form b = newService(sf);
        Form bBindB = b.bind("voidAboolean");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form c = newService(sf);
        Form cBindC = c.bind("voidAshort");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form d = newService(sf);
        Form dBindD = d.bind("voidAbyte");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form e = newService(sf);
        Form eBindE = e.bind("voidAlong");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form f = newService(sf);
        Form fBindF = f.bind("voidAdouble");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form g = newService(sf);
        Form gBindG = g.bind("voidAfloat");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form h = newService(sf);
        Form hBindH = h.bind("voidAchar");

		sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form i = newService(sf);
        Form iBindI = i.bind("voidAObject");

        sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form x = newService(sf);
        Form xBindX = x.bind("initialize");

        Form ext = aBindA.extend(bBindB).extend(cBindC).extend(dBindD);
		ext = ext.extend(eBindE).extend(fBindF).extend(gBindG);
		ext = ext.extend(hBindH).extend(iBindI).extend(jBindJ).extend(xBindX);

		Object obj = this.instantiateInterface(aClass, ext);

		// Now let's test:
		// This is a strange test, if we can call the method, we know that the class is wellformed
		((VoidReturnInterfaceTest) obj).voidAvoid();
		((VoidReturnInterfaceTest) obj).voidAint(42);
		((VoidReturnInterfaceTest) obj).voidAboolean(true);
		((VoidReturnInterfaceTest) obj).voidAshort((short) 42);
		((VoidReturnInterfaceTest) obj).voidAbyte((byte) 42);
		((VoidReturnInterfaceTest) obj).voidAlong(42);
		((VoidReturnInterfaceTest) obj).voidAdouble((double) 42);
		((VoidReturnInterfaceTest) obj).voidAfloat((float) 42);
		((VoidReturnInterfaceTest) obj).voidAchar('a');
		((VoidReturnInterfaceTest) obj).voidAObject("Finished!");
	}

	public void testInterfaceMathOperationTest() {
		String pkgName = "ch.unibe.piccola.bytecode.";
		bcf.generateInterface(pkgName + "IMathOperationsTest");
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		sf.clear();
		sf.emitJavaObject(null, new Boolean(false));
		sf.emitExit(null);
		Form a = newService(sf);
        Form aBindA = a.bind("isZero");

		sf.clear();
		sf.emitJavaObject(null, new Integer(2));
		sf.emitExit(null);
		Form b = newService(sf);
        Form bBindB = b.bind("inc");

     	sf.clear();
		sf.emitJavaObject(null, new Integer(3));
		sf.emitExit(null);
		Form c = newService(sf);
        Form cBindC = c.bind("add");

     	sf.clear();
		sf.emitJavaObject(null, new Integer(2));
		sf.emitExit(null);
		Form d = newService(sf);
        Form dBindD = d.bind("sub");

     	sf.clear();
		sf.emitJavaObject(null, new Integer(42));
		sf.emitExit(null);
		Form e = newService(sf);
        Form eBindE = e.bind("mult");

     	sf.clear();
		sf.emitJavaObject(null, new Integer(7));
		sf.emitExit(null);
		Form f = newService(sf);
        Form fBindF = f.bind("div");

        sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form x = newService(sf);
        Form xBindX = x.bind("initialize");

		Form ext = aBindA.extend(bBindB).extend(cBindC).extend(dBindD);
		ext = ext.extend(eBindE).extend(fBindF).extend(xBindX);

		Object obj = this.instantiateInterface(aClass, ext);

		//Now let's test:
		assertEquals(false, ((IMathOperationsTest) obj).isZero(1));
		assertEquals(2, ((IMathOperationsTest) obj).inc(1));
		assertEquals(3, ((IMathOperationsTest) obj).add(1, 2));
		assertEquals(2, ((IMathOperationsTest) obj).sub(3, 1));
		assertEquals(42, ((IMathOperationsTest) obj).mult(6, 7));
		assertEquals(7, ((IMathOperationsTest) obj).div(42, 6));
	}

	public void testMultipleInterfaces() {
		ArrayList multipleInterfaces = new ArrayList();
		multipleInterfaces.add("java.lang.Comparable");
		multipleInterfaces.add("java.lang.reflect.Member");

		bcf.generateInterfaces(multipleInterfaces);
		Class aClass = bcf.loadClass();
		assertNotNull(aClass);

		String name = "GMember";
		int modifiers = 8;
		int compare = 0;
		Class declaringClass = null;
		try{
			declaringClass= Class.forName("java.lang.String");
		} catch (ClassNotFoundException cnfe) {cnfe.printStackTrace();}

		sf.emitJavaObject(null, name);
        sf.emitExit(null);
		Form a = newService(sf);
        Form aBindA = a.bind("getName");

		sf.clear();
		sf.emitJavaObject(null, new Integer(modifiers));
		sf.emitExit(null);
		Form b = newService(sf);
        Form bBindB = b.bind("getModifiers");

		sf.clear();
		sf.emitJavaObject(null, declaringClass);
		sf.emitExit(null);
		Form c = newService(sf);
        Form cBindC = c.bind("getDeclaringClass");

		sf.clear();
		sf.emitJavaObject(null, new Integer(compare));
		sf.emitExit(null);
		Form d = newService(sf);
        Form dBindD = d.bind("compareTo");

        sf.clear();
		sf.emitEmpty(null);
		sf.emitExit(null);
		Form x = newService(sf);
        Form xBindX = x.bind("initialize");

		Form ext = aBindA.extend(bBindB).extend(cBindC).extend(dBindD).extend(xBindX);

		Object obj = this.instantiateInterface(aClass, ext);

		//Now let's test:
		assertEquals(name, ((Member) obj).getName());
		assertEquals(modifiers, ((Member) obj).getModifiers());
		assertSame(declaringClass, ((Member) obj).getDeclaringClass());
		assertEquals(compare, ((Comparable) obj).compareTo(name));
	}

	public void testAWTEvents() {
		String pkgName = "java.awt.event.";
		assertNotNull(generateAndLoadInterface(pkgName + "ActionListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "AdjustmentListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "AWTEventListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "ComponentListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "ContainerListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "FocusListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "HierarchyBoundsListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "HierarchyListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "InputMethodListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "ItemListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "KeyListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "MouseListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "MouseMotionListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TextListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "WindowListener"));
	}

	public void testSwingEventListeners() {
		String pkgName = "javax.swing.event.";
		assertNotNull(generateAndLoadInterface(pkgName + "AncestorListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "CaretListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "CellEditorListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "ChangeListener"));
//		bcf.generateAndLoad("GDocumentEvent", pkgName + "DocumentEvent");
//		bcf.generateAndLoad("GDocumentEvent.ElementChange", pkgName + DocumentEvent.ElementChange");
		assertNotNull(generateAndLoadInterface(pkgName + "DocumentListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "HyperlinkListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "InternalFrameListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "ListDataListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "ListSelectionListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "MenuDragMouseListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "MenuKeyListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "MenuListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "MouseInputListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "PopupMenuListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TableColumnModelListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TableModelListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TreeExpansionListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TreeModelListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TreeSelectionListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "TreeWillExpandListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "UndoableEditListener"));
	}

	public void testBeansEventListeners() {
		String pkgName = "java.beans.";
		assertNotNull(generateAndLoadInterface(pkgName + "PropertyChangeListener"));
		assertNotNull(generateAndLoadInterface(pkgName + "VetoableChangeListener"));
	}

	private Class generateAndLoadInterface(String className) {
		bcf.generateInterface(className);
		return bcf.loadClass();
	}

	private Object instantiateInterface(Class aClass, Form arg) {
		//System.err.println("Form ext: " + ext);
		// Whoever said that writing tests is fun, was wrong.
		Constructor cstr = null;
		Class par1 = null;
		try {
			par1 = Class.forName(FORM_CLASS);
		} catch (ClassNotFoundException cnfe) {cnfe.printStackTrace();}
		Class[] parameterTypes = {par1};
		try {
			cstr= aClass.getConstructor(parameterTypes);
		} catch (NoSuchMethodException nsme) {nsme.printStackTrace();}
		Object[] initArgs = {arg};

		Object obj = null;
		try {
			obj = cstr.newInstance(initArgs);
		} catch (Exception e) {e.printStackTrace();}

		return obj;
	}
}
