package ch.unibe.piccola.bytecode;

import ch.unibe.piccola.PiccolaException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/*
 * Inspired by "Understanding the Java ClassLoader"
 */

public class CompilingClassLoader extends ClassLoader { 
	public Class compileJavaFileAndLoadIntoVM(String fileName) {
		Class loadedClass = null;
		try {
			loadedClass = loadClass(fileName);
		} 
		catch (ClassNotFoundException e){
			throw new PiccolaException("Bridge: Compiling Class Loader exception: Class not found");
		}
		return loadedClass;
	}
	
	public Class compileJavaSourceAndLoadIntoVM(String sourceCode, String className) {
		//Save the file,
		String fileName = className + ".java";
		try {
			FileOutputStream out = new FileOutputStream( fileName );
			byte[] bytes = sourceCode.getBytes();
			out.write(bytes);
			out.close();		
		} 
		catch (java.io.IOException ie) {ie.toString();}
		Class loadedClass = null;
		// compile and load it,
		try {
			loadedClass = loadClass(className);
		}
		catch (ClassNotFoundException e){
			throw new PiccolaException("Bridge: Compiling Class Loader exception: Class not found");
		}
		// and delete it again
		File f = new File(className + ".java");
		f.delete();
		f = new File(className + ".class");
		f.delete();
		
		return loadedClass;
	}

	private boolean compileJavaFile(String fileStub ) throws IOException {
		System.out.println( "CCL: Compiling " + fileStub);
	
		Process p = null;
		try { p = Runtime.getRuntime().exec( "javac " + fileStub + ".java" );
		} catch (java.io.IOException e){ System.out.println( e ); }
		/*
		PrintWriter out = new PrintWriter(p.getOutputStream());
		System.out.println(p.getOutputStream().toString());
		System.out.println(out);
		out.flush();
		*/
		// Wait for it to finish running 
		try { p.waitFor(); 
		} catch( InterruptedException ie ) { System.out.println( ie ); }
	
		int ret = p.exitValue();
		boolean success = (ret==0);
		if (success==true) {
			System.out.println("Compilation success.");
		} else {
			System.out.println("Compilation failed.");
		}
		return success;
	}
	
	public Class loadClass( String name, boolean resolve ) throws ClassNotFoundException {
		Class aClass = null;
	
		// First, see if we've already dealt with this one 
		aClass = findLoadedClass( name );

		String fileStub = name.replace( '.', '/' ); 
		
		String javaFilename = fileStub+".java";
		String classFilename = fileStub+".class";
		File javaFile = new File( javaFilename );
		File classFile = new File( classFilename );
		
		if (javaFile.exists()) { 
			try {
				if (!compileJavaFile( fileStub )) {
					throw new ClassNotFoundException( "Compile failed: "+javaFilename ); 
				}
			} catch( IOException ie ) {
				throw new ClassNotFoundException( ie.toString() ); } 
		}

		try {
			byte raw[] = getBytes( classFilename );
			aClass = defineClass( name, raw, 0, raw.length ); } 
		catch( IOException ie ) {
			// This is not a failure! If we reach here, it might mean that we are dealing with a class in a library,
			// such as java.lang.Object 
		}
		// Maybe the aClasss is in a library -- try loading the normal way
		if (aClass==null)
			aClass = findSystemClass( name );
		
		// Resolve the aClasss, if any, but only if the "resolve" flag is set to true
		if (resolve && aClass != null)
			resolveClass( aClass );

		if (aClass == null) {
			throw new ClassNotFoundException( name );
		} else {
			System.out.println("Loading success.");
		}
		return aClass;
	}

	private byte[] getBytes( String filename ) throws IOException { 
		File file = new File( filename );
		long len = file.length();
		byte raw[] = new byte[(int)len]; 
		
		FileInputStream fin = new FileInputStream( file ); 
		
		int r = fin.read( raw ); 
		if (r != len) 
			throw new IOException( "Can't read all, "+r+" != "+len );
		fin.close();
		
		return raw; 
	}
}
