package ch.unibe.piccola.bytecode;

import java.io.ByteArrayOutputStream;
import java.lang.reflect.Method;
import java.util.ArrayList;

import org.apache.bcel.Constants;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.Type;
import org.apache.bcel.generic.PUSH;
import org.apache.bcel.generic.DUP;

/**
 * @author  $Id: ConstructorFactory.java,v 3.5 2003/01/10 13:46:25 kneubuhl Exp $
**/

public class ConstructorFactory implements Constants, FactoryConstants {

	private String m_initiationLabel = "initialize";

	private InstructionFactory m_factory;

	private ClassGen m_classGen;

	public ConstructorFactory(ClassGen cg) {
		m_classGen = cg;
		m_factory = new InstructionFactory(cg);
	}

	public ClassGen addConstructor(String className, String superClassName) {
		addVoidConstructor(className, superClassName);
		addDelegateConstructor(className, superClassName);
		addInit(className, superClassName);
		addSetHandler(className, superClassName);
		return m_classGen;
	}

	public void setInitiationLabel(String label) {
		m_initiationLabel = label;
	}

	protected void addVoidConstructor(String className, String superClassName) {
    	InstructionList il = new InstructionList();
		ConstantPoolGen cp = m_classGen.getConstantPool();
		MethodGen mg = new MethodGen(ACC_PUBLIC, Type.VOID, Type.NO_ARGS ,
									 null, "<init>", className, il, cp);

		// Generate SuperCall
		il = generateSuperCall(il, superClassName);
		il.append(m_factory.createReturn(Type.VOID));

		mg.setMaxStack();
		m_classGen.addMethod(mg.getMethod());
		il.dispose();
    }


    protected void addDelegateConstructor(String className, String superClassName) {
    	InstructionList il = new InstructionList();
		ConstantPoolGen cp = m_classGen.getConstantPool();
		MethodGen mg = new MethodGen(ACC_PUBLIC, Type.VOID, new Type[]{m_IFormObjectType},
									 new String[] { "delegate" },
									 "<init>", className, il, cp);

		// Generate SuperCall
		il = generateSuperCall(il, superClassName);

		// Create setHandler call
		il.append(m_factory.createThis());
		il.append(m_factory.createLoad(Type.OBJECT,1));
		il.append(m_factory.createInvoke(className, "setHandler",
					       	Type.VOID, new Type[]{m_IFormObjectType},
				    	   	Constants.INVOKEVIRTUAL));

		il.append(m_factory.createReturn(Type.VOID));

		mg.setMaxStack();
		m_classGen.addMethod(mg.getMethod());
		il.dispose();
    }

	protected void addSetHandler(String className, String superClassName) {
		InstructionList il = new InstructionList();
		ConstantPoolGen cp = m_classGen.getConstantPool();
		MethodGen mg = new MethodGen(ACC_PUBLIC, Type.VOID, new Type[]{m_IFormObjectType},
									 null, "setHandler", className, il, cp);

		il = generateSetDelegateInstructions(il, className);

		il = generateInitCall(il, className);

		il.append(m_factory.createReturn(Type.VOID));

		mg.setMaxStack();
		m_classGen.addMethod(mg.getMethod());
		il.dispose();
	}

	protected void addInit(String className, String superClassName) {
		InstructionList il = new InstructionList();
		ConstantPoolGen cp = m_classGen.getConstantPool();
		MethodGen mg = new MethodGen(ACC_PRIVATE, Type.VOID, Type.NO_ARGS,
									 null, m_initiationLabel, className, il, cp);
		/* Create Arguments */
		il.append(m_factory.createNew(ARGUMENTS_CLASS));
		il.append(new DUP());
		il.append(m_factory.createInvoke(ARGUMENTS_CLASS, "<init>",
											Type.VOID, Type.NO_ARGS,
											Constants.INVOKESPECIAL));
		il.append(m_factory.createStore(Type.OBJECT, 1));
		il.append(m_factory.createLoad(Type.OBJECT, 1));

		// add this
		il.append(m_factory.createThis());
		il.append(m_factory.createInvoke(ARGUMENTS_CLASS, "add",
											Type.VOID, new Type[] {Type.OBJECT},
										    Constants.INVOKEVIRTUAL));
		// call to extractHander
		il.append(m_factory.createThis());
		il.append(new PUSH(cp, m_initiationLabel));
		il.append(m_factory.createInvoke(className, "getService",
				       new ObjectType(FORM_CLASS),
				       new Type[] {new ObjectType("java.lang.String")},
				       Constants.INVOKEVIRTUAL));
		// call to callback
		il.append(m_factory.createLoad(Type.OBJECT, 1));
		il.append(m_factory.createInvoke(FORM_CLASS, "callback",
			   Type.OBJECT, new Type[] {new ObjectType(ARGUMENTS_CLASS)},
			   Constants.INVOKEVIRTUAL));

		il.append(m_factory.createPop(1));
		il.append(m_factory.createReturn(Type.VOID));

		mg.setMaxStack();
		mg.setMaxLocals();
		m_classGen.addMethod(mg.getMethod());
		il.dispose();
	}

	private InstructionList generateSuperCall(InstructionList il, String superClassName) {
		il.append(m_factory.createThis());
		il.append(m_factory.createInvoke(superClassName, "<init>",
									   	Type.VOID, Type.NO_ARGS,
									   	Constants.INVOKESPECIAL));
		return il;
	}

	private InstructionList generateSetDelegateInstructions(InstructionList il, String className) {
		il.append(m_factory.createThis());
        il.append(m_factory.createLoad(m_IFormObjectType,1));
        il.append(m_factory.createPutField(className, "m_delegate", m_IFormObjectType));
		return il;
	}

	private InstructionList generateInitCall(InstructionList il, String className) {
		il.append(m_factory.createThis());
		il.append(m_factory.createInvoke(className, m_initiationLabel,
					       	Type.VOID, Type.NO_ARGS,
				    	   	Constants.INVOKEVIRTUAL));
		return il;
	}
}
