package ch.unibe.piccola.code;

import ch.unibe.piccola.source.SourceRef;

/**
 * An opcode for the Piccola virtual machine.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Aug 08, 2002)
**/
public abstract class OpCode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Compares the specified object with this opcode for equality. Returns
     * <code>true</code> if and only if the specified opcode is of the same
     * class as this opcode and has the same parameters.
     *
     * @param   object the object to be compared for equality with this opcode.
     * @return  <code>true</code> if the specified object is equal to this
     *          opcode.
    **/
    public boolean equals(Object object)
    {
        if (null == object) {
            return false;
        }
        return getClass().equals(object.getClass());
    }

    /**
     * Executes this opcode on the specified Piccola virtual machine.
     *
     * @param   vm the virtual machine on which this opcode will be executed.
    **/
    public abstract void execute(IVM vm);


    /**
     * Returns the source code reference for this opcode.
     *
     * @return  the source code reference for this opcode.
    **/
    public final SourceRef getSourceRef()
    {
        return m_sourceRef;
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    OpCode(SourceRef sourceRef)
    {
        m_sourceRef = sourceRef;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final SourceRef m_sourceRef;

}