package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a constant syntax node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class Constant extends LeafNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitConstant method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitConstant(this);
    }

    /**
     * Compares this constant node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this constant node is equal to object,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        Constant other = (Constant) object;
        return m_constant.equals(other.m_constant);
    }

    /**
     * Returns the constant object represented by this node.
    **/
    public Object getConstant()
    {
        return m_constant;
    }

    /**
     * Returns a debug string representation of this constant node.
     *
     * @return a debug string representation of this constant node.
    **/
    public String toString()
    {
        return "Constant(" + m_constant + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    Constant(SourceRef sourceRef, Object constant)
    {
        super(sourceRef);
        m_constant = constant;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Object m_constant;

}