package ch.unibe.piccola.syntax;

import java.util.Arrays;
import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a curried abstraction syntax node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class CurriedAbstraction extends OneChildNode
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Accepts a node visitor.
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visitCurriedAbstraction method.
    **/
    public Object accept(NodeVisitor visitor)
    {
        return visitor.visitCurriedAbstraction(this);
    }

    /**
     * Compares this curried abstraction node with the specified object.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this curried abstraction node is equal to
     *         object, <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (!super.equals(object)) {
            return false;
        }
        CurriedAbstraction other = (CurriedAbstraction) object;
        return Arrays.equals(m_parameters, other.m_parameters);
    }

    /**
     * Returns the parameters of this curried abstraction node.
     *
     * @return the parameters of this curried abstraction node.
    **/
    public String[] getParameters()
    {
        return m_parameters;
    }

    /**
     * Returns a debug string representation of this curried abstraction node.
     *
     * @return a debug string representation of this curried abstraction node.
    **/
    public String toString()
    {
        return "CurriedAbstraction(" + parametersString(m_parameters) + ", " +
            getChild() + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    CurriedAbstraction(SourceRef sourceRef, Node child, String[] parameters)
    {
        super(sourceRef, child);
        m_parameters = parameters;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Node newInstance(Node child)
    {
        return new CurriedAbstraction(getSourceRef(), child, m_parameters);
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String[] m_parameters;

}