package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * Represents a Piccola syntax node.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public abstract class Node
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    /**
     * Hook method for node visitor. Calls visitor.visit[NodeType](this).
     *
     * @param  visitor the node visitor to be accepted.
     *
     * @return the result of the visitor's visit method corresponding to
     *         the node.
    **/
    public abstract Object accept(NodeVisitor visitor);

    /**
     * Defines equality for nodes. Main usage for tests.
     *
     * @param  object the object to be compared with this node.
     *
     * @return <code>true</code> if this node is equal to object,
     *         <code>false</code> otherwise.
    **/
    public boolean equals(Object object)
    {
        if (null == object) {
            return false;
        }
        return getClass().equals(object.getClass());
    }

    /**
     * Returns the source code reference of this node.
     *
     * @return   the source code reference of this node.
    **/
    public SourceRef getSourceRef()
    {
        return m_sourceRef;
    }

    /**
     * Returns a debug String representation of this node.
     *
     * @return   a debug String representation of this node.
    **/
    public abstract String toString();

    public abstract Node transform(NodeVisitor visitor);

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Node(SourceRef sourceRef)
    {
        m_sourceRef = sourceRef;
    }

    protected String labelsString(boolean fixedPoint, String[] labels)
    {
        StringBuffer result = new StringBuffer();
        if (fixedPoint) {
            result.append("def");
        }
        result.append(labels[0]);
        for (int i = 1; i < labels.length; ++i) {
            result.append("." + labels[i]);
        }
        return result.toString();
    }

    protected String parametersString(String[] parameters)
    {
        StringBuffer result = new StringBuffer();
        result.append(null == parameters[0] ? "()" : parameters[0]);
        for (int i = 1; i < parameters.length; ++i) {
            result.append(" ");
            result.append(null == parameters[i] ? "()" : parameters[i]);
        }
        return result.toString();
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final SourceRef m_sourceRef;

}

