package ch.unibe.piccola.syntax;

import ch.unibe.piccola.source.SourceRef;

/**
 * A node visitor that visits only simplified Piccola syntax nodes. If a
 * not simplified node is encountered, the visitor throws an Error.
 *
 * @author   Stefan Kneubuehl
 * @version  3.4c (Nov 14, 2002)
**/
public class SimplifiedNodeVisitor extends NodeVisitor
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public final Object visitCollection(Collection node)
    {
        return invalidNode("Collection");
    }

    public final Object visitCompoundBinding(CompoundBinding node)
    {
        return invalidNode("CompoundBinding");
    }

    public final Object visitCurriedAbstraction(CurriedAbstraction node)
    {
        return invalidNode("CurriedAbstraction");
    }

    public final Object visitInfixApplication(InfixApplication node)
    {
        return invalidNode("InfixApplication");
    }

    public final Object visitNamedAbstraction(NamedAbstraction node)
    {
        return invalidNode("NamedAbstraction");
    }

    public final Object visitPrefixApplication(PrefixApplication node)
    {
        return invalidNode("PrefixApplication");
    }

    public final Object visitQuote(Quote node)
    {
        return invalidNode("Quote");
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Object invalidNode(String nodeName)
    {
        throw new RuntimeException(getClass().getName() + " does not support "
            + nodeName + " syntax node.");
    }

}