package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * Represents an provided extension type 'P.Q'.
 *
 * @author    Stefan Kneubuehl
 * @version   3.4c (Jan 03, 2003)
**/
public class TypeExtension extends Type
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        if (object instanceof TypeExtension) {
            TypeExtension other = (TypeExtension) object;
            return m_left.equals(other.m_left) &&
                   m_right.equals(other.m_right) &&
                   getClass().equals(other.getClass());
        }
        return false;
    }

    public Type getLeft()
    {
        return m_left;
    }

    public Type getRight()
    {
        return m_right;
    }

    public boolean occurs(TypeVariable var)
    {
        return m_left.occurs(var) || m_right.occurs(var);
    }

    public Type reverse()
    {
        return doExtend(m_left.reverse(), m_right.reverse());
    }

    public Type subst(TypeVariable var, Type replace)
    {
        return doExtend(m_left.subst(var, replace), m_right.subst(var, replace));
    }

    public String toString()
    {
        return "(" + m_left + "," + m_right + ")";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    public TypeExtension(Type left, Type right)
    {
        m_left = left;
        m_right = right;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Type lookupLabel(String x)
    {
        if (m_right.isVariable()) {
            return null;
        }
        Type result = m_right.lookupLabel(x);
        if (null == result) {
            result = m_left.lookupLabel(x);
        }
        return result;
    }

    protected TypeService lookupService()
    {
        if (m_right.isVariable()) {
            return null;
        }
        TypeService result = m_right.lookupService();
        if (null == result) {
            result = m_left.lookupService();
        }
        return result;
    }

    protected Type removeBindings()
    {
        return doExtend(m_right.isVariable() ? m_left : m_left.removeBindings(),
                        m_right.removeBindings());
    }

    protected Type removeLabel(String x)
    {
        return doExtend(m_right.isVariable() ? m_left : m_left.removeLabel(x),
                        m_right.removeLabel(x));
    }

    protected Type removeService()
    {
        return doExtend(m_left.removeService(), m_right.removeService());
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Type m_left;

    private final Type m_right;

    private Type doExtend(Type left, Type right)
    {
        return (m_left == left && m_right == right) ? this : left.extend(right);
    }

}