package ch.unibe.piccola.type;

import java.util.ArrayList;

/**
 * @author   Stefan Kneubuehl
 * @version  3.4c (Jan 04, 2003)
 *
 * Represents the provided type T\x
**/
public class TypeHide extends Type
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public boolean equals(Object object)
    {
        if (object instanceof TypeHide) {
            TypeHide other = (TypeHide) object;
            return m_label.equals(other.m_label) &&
                   m_type.equals(other.m_type);
        }
        return false;
    }

    public String getLabel()
    {
        return m_label;
    }

    /**
     * Implements type equivalences '(P\x)\x = (P\x)' and '(P\y)\x = (P\x)\y'.
    **/
    public Type hide(String x)
    {
        return m_label.equals(x) ? this : doHide(m_type.hide(x));
    }

    public boolean isVariable()
    {
        return m_type.isVariable();
    }

    public boolean occurs(TypeVariable var)
    {
        return m_type.occurs(var);
    }

    public Type reverse()
    {
        return doHide(m_type.reverse());
    }

    public Type subst(TypeVariable var, Type replace)
    {
        return doHide(m_type.subst(var, replace));
    }

    public String toString()
    {
        return m_type.toString() + "\\" + m_label;
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    TypeHide(String label, Type type)
    {
        m_label = label;
        m_type = type;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected Type lookupLabel(String x)
    {
        return m_label.equals(x) ? super.lookupLabel(x) : m_type.lookupLabel(x);
    }

    protected TypeService lookupService()
    {
        return m_type.lookupService();
    }

    protected Type removeBindings()
    {
        return doHide(m_type.removeBindings());
    }

    protected Type removeLabel(String x)
    {
        return doHide(m_type.removeLabel(x));
    }

    protected Type removeService()
    {
        return doHide(m_type.removeService());
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final String m_label;

    private final Type m_type;

    private Type doHide(Type type)
    {
        return (m_type == type) ? this : type.hide(m_label);
    }

}