package ch.unibe.piccola.vm;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.SortedMap;
import java.util.TreeMap;
import ch.unibe.piccola.syntax.Syntax;

/**
 * @author   Stefan Kneubuehl
 * @version  4.0 (Jun 01, 2003)
**/
public final class JavaClass
{

    // ------------------------------------------------------------------------
    // public static
    // ------------------------------------------------------------------------

    public synchronized static JavaClass lookup(Class cl)
    {
        if (!m_instances.containsKey(cl)) {
            m_instances.put(cl, new JavaClass(cl));
        }
        return (JavaClass) m_instances.get(cl);
    }

    public static JavaClass lookup(String className) throws ClassNotFoundException
    {
        Class cl;
        if (className.equals("boolean")) { cl = Boolean.TYPE; }
        else if (className.equals("byte")) { cl = Byte.TYPE; }
        else if (className.equals("char")) { cl = Character.TYPE; }
        else if (className.equals("double")) { cl = Double.TYPE; }
        else if (className.equals("float")) { cl = Float.TYPE; }
        else if (className.equals("int")) { cl = Integer.TYPE; }
        else if (className.equals("long")) { cl = Long.TYPE; }
        else if (className.equals("short")) { cl =  Short.TYPE; }
        else cl = Class.forName(className);
        return lookup(cl);
    }

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final String LABEL_NEW = Syntax.labelNew();

    private static final String LABEL_NEWARRAY = Syntax.labelNewArray();

    private static final HashMap m_instances = new HashMap();

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public SortedMap getClassAttributes()
    {
        return m_classAts;
    }

    public String getClassName()
    {
        return m_class.getName();
    }

    public SortedMap getClassOperations()
    {
        return m_classOps;
    }

    public SortedMap getInstanceAttributes()
    {
        return m_instanceAts;
    }

    public SortedMap getInstanceOperations()
    {
        return m_instanceOps;
    }


    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Class m_class;

    private final SortedMap m_classAts;

    private final SortedMap m_classOps;

    private final SortedMap m_instanceAts;

    private final SortedMap m_instanceOps;

    private JavaClass(Class cl)
    {
        m_class = cl;
        m_classAts = new TreeMap();
        m_classOps = new TreeMap();
        m_instanceAts = new TreeMap();
        m_instanceOps = new TreeMap();
        initAttributes();
        initOperations();
    }

    private void addMethod(SortedMap map, Method method)
    {
        String label = method.getName();
        JavaMethod op = (JavaMethod) map.get(label);
        if (null == op) {
            map.put(label, new JavaMethod(method));
        }
        else {
            op.add(method);
        }
    }

    private void initAttributes()
    {
        Field[] fields = m_class.getFields();
        for (int i = 0; i < fields.length; ++i) {
            if (Modifier.isFinal(fields[i].getModifiers())) {
                if (Modifier.isStatic(fields[i].getModifiers())) {
                    m_classAts.put(fields[i].getName(), fields[i]);
                }
                else {
                    m_instanceAts.put(fields[i].getName(), fields[i]);
                }
            }
        }
    }

    private void initOperations()
    {
        if (m_class.getConstructors().length > 0) {
            m_classOps.put(LABEL_NEW, new JavaConstructor(m_class.getConstructors()));
        }
        m_classOps.put(LABEL_NEWARRAY, new JavaArrayConstructor(m_class));
        Method[] methods = m_class.getMethods();
        for (int i = 0; i < methods.length; ++i) {
            if (Modifier.isStatic(methods[i].getModifiers())) {
                addMethod(m_classOps, methods[i]);
            }
            else {
                addMethod(m_instanceOps, methods[i]);
            }
        }
    }

}
