package ch.unibe.piccola.vm;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Constructor;
import ch.unibe.piccola.code.Arguments;

/**
 * Piccola services that map to constructor methods.
 *
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public class JavaConstructor extends JavaOperation
{

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public Object call(Object target, Arguments args)
    {
        try {
            return m_ctors[findMatch(args)].newInstance(args.toArray());
        }
        catch (IllegalAccessException ex) {
            throw new AgentException(ex);
        }
        catch (IllegalArgumentException ex) {
            throw new AgentException(ex);
        }
        catch (InstantiationException ex) {
            throw new AgentException(ex);
        }
        catch (InvocationTargetException ex) {
            throw new AgentException(ex.getTargetException());
        }
    }

    public boolean equals(Object other)
    {
        return this == other;
    }

    public int hashCode()
    {
        return m_ctors[0].hashCode();
    }

    public String toString()
    {
        return m_ctors[0].getDeclaringClass().getName() + ".new";
    }

    // ------------------------------------------------------------------------
    // default
    // ------------------------------------------------------------------------

    JavaConstructor(Constructor[] ctors)
    {
        m_ctors = ctors;
    }

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    protected int count()
    {
        return m_ctors.length;
    }

    protected Class[] paramTypes(int index)
    {
        return m_ctors[index].getParameterTypes();
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Constructor[] m_ctors;

}

