package ch.unibe.piccola.vm;

import java.util.Arrays;
import java.util.ArrayList;
import ch.unibe.piccola.code.Arguments;
import ch.unibe.piccola.code.JavaOp;

/**
 * Piccola services that map to Java methods.
 *
 * @author   Stefan Kneubuehl
 * @version  3.6c (May 11, 2003)
**/
public abstract class JavaOperation extends JavaOp
{

    // ------------------------------------------------------------------------
    // protected
    // ------------------------------------------------------------------------

    /**
     * Executes this Java operation with the specified arguments on the
     * specified target.
     *
     * @param  target the target of the service call.
     * @param  args the arguments for the call.
     * @return the return value of the service call.
    **/
    public abstract Object call(Object target, Arguments args);

    protected abstract int count();

    protected abstract Class[] paramTypes(int index);

    protected int findMatch(Arguments args)
    {
        if (1 == count()) { return 0; }
        int match = -1;
        Class[] argTypes = args.argumentTypes();
        for (int i = 0; i < count(); ++i) {
            if (matchExact(paramTypes(i), argTypes)) {
                return i;
            }
            if (match(paramTypes(i), argTypes)) {
                if (-1 == match) {
                    match = i;
                }
                else {
                    throw new AgentException("Ambiguous Java call");
                }
            }
        }
        if (-1 == match) {
            System.out.println("Checked " + count() + " methods");
            System.out.println("ARG TYPES = " + asString(argTypes));
            for (int i = 0; i < count(); ++i) {
                System.out.println("PARAM TYPES " + i + ": " + asString(paramTypes(i)));
            }
            throw new AgentException("No matching method found for " + this +
                "(" + asString(argTypes) + ")");
        }
        return match;
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private boolean isAssignable(Class target, Class source)
    {
        return null == source ? !target.isPrimitive() :
           paramType(target).isAssignableFrom(paramType(source));
    }

    private boolean match(Class[] paramTypes, Class[] argTypes)
    {
        if (paramTypes.length != argTypes.length) {
            return false;
        }
        for (int i = 0; i < paramTypes.length; ++i) {
            if (!isAssignable(paramTypes[i], argTypes[i])) {
                return false;
            }
        }
        return true;
    }

    private boolean matchExact(Class[] paramTypes, Class[] argTypes)
    {
        if (paramTypes.length != argTypes.length) {
            return false;
        }
        for (int i = 0; i < paramTypes.length; ++i) {
            if (!paramType(paramTypes[i]).equals(paramType(argTypes[i]))) {
                return false;
            }
        }
        return true;
    }

    private String asString(Object[] arr)
    {
        StringBuffer result = new StringBuffer();
        if (0 < arr.length) {
            result.append(arr[0]);
        }
        for (int i = 1; i < arr.length; ++i) {
            result.append(", ");
            result.append(arr[i]);
        }
        return result.toString();
    }

    private Class paramType(Class cl)
    {
        if (null == cl) return cl;
        if (cl.equals(Boolean.TYPE))   return Boolean.class;
        if (cl.equals(Byte.TYPE))      return Byte.class;
        if (cl.equals(Character.TYPE)) return Character.class;
        if (cl.equals(Double.TYPE))    return Double.class;
        if (cl.equals(Float.TYPE))     return Float.class;
        if (cl.equals(Integer.TYPE))   return Integer.class;
        if (cl.equals(Long.TYPE))      return Long.class;
        if (cl.equals(Short.TYPE))     return Short.class;
        return cl;
    }

}
