package ch.unibe.piccola.vm;

import ch.unibe.piccola.code.Arguments;
import ch.unibe.piccola.code.Code;
import ch.unibe.piccola.syntax.Syntax;

/**
 * @author   Stefan Kneubuehl
 * @version  4.0 (Jun 01, 2003)
**/
public class PiccolaService extends Service
{

    // ------------------------------------------------------------------------
    // private static
    // ------------------------------------------------------------------------

    private static final String LABEL_ARG = Syntax.labelArg();

    // ------------------------------------------------------------------------
    // public
    // ------------------------------------------------------------------------

    public PiccolaService(VMPool pool, Code code, Form context)
    {
        m_code = code;
        m_context = context;
        m_pool = pool;
    }

    public Object callback(Arguments args)
    {
        Form dynamic = m_pool.getCallDynamic();
        dynamic = dynamic.extend(m_pool.up(args).bind(LABEL_ARG));
        return evaluate(dynamic).down();
    }

    public Form evaluate(Form dynamic)
    {
        VM vm = new VM(m_pool, m_code, m_context, dynamic);
        return vm.evaluate();
    }

    public Code getCode()
    {
        return m_code;
    }

    public Form getContext()
    {
        return m_context;
    }

    public void run(Form dynamic)
    {
        VM vm = new VM(m_pool, m_code, m_context, dynamic);
        vm.start();
    }

    // ------------------------------------------------------------------------
    // private
    // ------------------------------------------------------------------------

    private final Code m_code;

    private final Form m_context;

    private final VMPool m_pool;

    private Form upArguments(Arguments args)
    {
        if (args.size() == 0) {
            return m_pool.m_factory.emptyForm();
        }
        if (args.size() == 1) {
            return m_pool.up(args.get(0));
        }
        Form result = m_pool.m_factory.emptyForm();
        for (int i = 0; i < args.size(); ++i) {
            result = result.extend(m_pool.up(args.get(0)).bind(Syntax.labelVal(i)));
        }
        return result;
    }

}
